@@ -6,24 +6,8 @@ SPECS = {
     </method>
   </interface>
 """,
-    "org.storage.stratis2.FetchProperties.r4": """
-<interface name="org.storage.stratis2.FetchProperties.r4">
-    <method name="GetAllProperties">
-      <arg name="results" type="a{s(bv)}" direction="out" />
-    </method>
-    <method name="GetProperties">
-      <arg name="properties" type="as" direction="in" />
-      <arg name="results" type="a{s(bv)}" direction="out" />
-    </method>
-  </interface>
-""",
-    "org.storage.stratis2.Manager.r4": """
-<interface name="org.storage.stratis2.Manager.r4">
-    <method name="ConfigureSimulator">
-      <arg name="denominator" type="u" direction="in" />
-      <arg name="return_code" type="q" direction="out" />
-      <arg name="return_string" type="s" direction="out" />
-    </method>
+    "org.storage.stratis3.Manager.r3": """
+<interface name="org.storage.stratis3.Manager.r3">
     <method name="CreatePool">
       <arg name="name" type="s" direction="in" />
       <arg name="redundancy" type="(bq)" direction="in" />
@@ -45,18 +29,32 @@ SPECS = {
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
+    <method name="ListKeys">
+      <arg name="result" type="as" direction="out" />
+      <arg name="return_code" type="q" direction="out" />
+      <arg name="return_string" type="s" direction="out" />
+    </method>
+    <method name="RefreshState">
+      <arg name="return_code" type="q" direction="out" />
+      <arg name="return_string" type="s" direction="out" />
+    </method>
     <method name="SetKey">
       <arg name="key_desc" type="s" direction="in" />
       <arg name="key_fd" type="h" direction="in" />
-      <arg name="interactive" type="b" direction="in" />
       <arg name="result" type="(bb)" direction="out" />
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
-    <method name="UnlockPool">
+    <method name="StartPool">
       <arg name="pool_uuid" type="s" direction="in" />
-      <arg name="unlock_method" type="s" direction="in" />
-      <arg name="result" type="(bas)" direction="out" />
+      <arg name="unlock_method" type="(bs)" direction="in" />
+      <arg name="result" type="(b(oaoao))" direction="out" />
+      <arg name="return_code" type="q" direction="out" />
+      <arg name="return_string" type="s" direction="out" />
+    </method>
+    <method name="StopPool">
+      <arg name="pool" type="o" direction="in" />
+      <arg name="result" type="(bs)" direction="out" />
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
@@ -66,13 +64,14 @@ SPECS = {
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
+    <property name="StoppedPools" type="a{sa{sv}}" access="read" />
     <property name="Version" type="s" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
   </interface>
 """,
-    "org.storage.stratis2.Report.r4": """
-<interface name="org.storage.stratis2.Report.r4">
+    "org.storage.stratis3.Report.r3": """
+<interface name="org.storage.stratis3.Report.r3">
     <method name="GetReport">
       <arg name="name" type="s" direction="in" />
       <arg name="result" type="s" direction="out" />
@@ -81,8 +80,8 @@ SPECS = {
     </method>
   </interface>
 """,
-    "org.storage.stratis2.blockdev.r4": """
-<interface name="org.storage.stratis2.blockdev.r4">
+    "org.storage.stratis3.blockdev.r3": """
+<interface name="org.storage.stratis3.blockdev.r3">
     <method name="SetUserInfo">
       <arg name="id" type="(bs)" direction="in" />
       <arg name="changed" type="(bs)" direction="out" />
@@ -98,6 +97,7 @@ SPECS = {
     <property name="InitializationTime" type="t" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
+    <property name="NewPhysicalSize" type="(bs)" access="read" />
     <property name="PhysicalPath" type="s" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
@@ -107,6 +107,7 @@ SPECS = {
     <property name="Tier" type="q" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="false" />
     </property>
+    <property name="TotalPhysicalSize" type="s" access="read" />
     <property name="UserInfo" type="(bs)" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="false" />
     </property>
@@ -115,8 +116,8 @@ SPECS = {
     </property>
   </interface>
 """,
-    "org.storage.stratis2.filesystem.r4": """
-<interface name="org.storage.stratis2.filesystem.r4">
+    "org.storage.stratis3.filesystem.r3": """
+<interface name="org.storage.stratis3.filesystem.r3">
     <method name="SetName">
       <arg name="name" type="s" direction="in" />
       <arg name="result" type="(bs)" direction="out" />
@@ -133,13 +134,15 @@ SPECS = {
     <property name="Pool" type="o" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
+    <property name="Size" type="s" access="read" />
+    <property name="Used" type="(bs)" access="read" />
     <property name="Uuid" type="s" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
   </interface>
 """,
-    "org.storage.stratis2.pool.r4": """
-<interface name="org.storage.stratis2.pool.r4">
+    "org.storage.stratis3.pool.r3": """
+<interface name="org.storage.stratis3.pool.r3">
     <method name="AddCacheDevs">
       <arg name="devices" type="as" direction="in" />
       <arg name="results" type="(bao)" direction="out" />
@@ -152,7 +155,7 @@ SPECS = {
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
-    <method name="Bind">
+    <method name="BindClevis">
       <arg name="pin" type="s" direction="in" />
       <arg name="json" type="s" direction="in" />
       <arg name="results" type="b" direction="out" />
@@ -166,7 +169,7 @@ SPECS = {
       <arg name="return_string" type="s" direction="out" />
     </method>
     <method name="CreateFilesystems">
-      <arg name="specs" type="as" direction="in" />
+      <arg name="specs" type="a(s(bs))" direction="in" />
       <arg name="results" type="(ba(os))" direction="out" />
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
@@ -177,12 +180,29 @@ SPECS = {
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
+    <method name="GrowPhysicalDevice">
+      <arg name="dev" type="s" direction="in" />
+      <arg name="results" type="b" direction="out" />
+      <arg name="return_code" type="q" direction="out" />
+      <arg name="return_string" type="s" direction="out" />
+    </method>
     <method name="InitCache">
       <arg name="devices" type="as" direction="in" />
       <arg name="results" type="(bao)" direction="out" />
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
+    <method name="RebindClevis">
+      <arg name="results" type="b" direction="out" />
+      <arg name="return_code" type="q" direction="out" />
+      <arg name="return_string" type="s" direction="out" />
+    </method>
+    <method name="RebindKeyring">
+      <arg name="key_desc" type="s" direction="in" />
+      <arg name="results" type="b" direction="out" />
+      <arg name="return_code" type="q" direction="out" />
+      <arg name="return_string" type="s" direction="out" />
+    </method>
     <method name="SetName">
       <arg name="name" type="s" direction="in" />
       <arg name="result" type="(bs)" direction="out" />
@@ -196,7 +216,7 @@ SPECS = {
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
-    <method name="Unbind">
+    <method name="UnbindClevis">
       <arg name="results" type="b" direction="out" />
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
@@ -206,10 +226,20 @@ SPECS = {
       <arg name="return_code" type="q" direction="out" />
       <arg name="return_string" type="s" direction="out" />
     </method>
+    <property name="AllocatedSize" type="s" access="read" />
+    <property name="AvailableActions" type="s" access="read" />
+    <property name="ClevisInfo" type="(b(b(ss)))" access="read" />
     <property name="Encrypted" type="b" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
+    <property name="FsLimit" type="t" access="readwrite" />
+    <property name="HasCache" type="b" access="read" />
+    <property name="KeyDescription" type="(b(bs))" access="read" />
     <property name="Name" type="s" access="read" />
+    <property name="NoAllocSpace" type="b" access="read" />
+    <property name="Overprovisioning" type="b" access="readwrite" />
+    <property name="TotalPhysicalSize" type="s" access="read" />
+    <property name="TotalPhysicalUsed" type="(bs)" access="read" />
     <property name="Uuid" type="s" access="read">
       <annotation name="org.freedesktop.DBus.Property.EmitsChangedSignal" value="const" />
     </property>
