@@ -20,7 +20,6 @@ import sys
 
 # isort: THIRDPARTY
 import dbus
-import psutil
 
 # isort: FIRSTPARTY
 from dbus_client_gen import (
@@ -28,17 +27,20 @@ from dbus_client_gen import (
     DbusClientMissingSearchPropertiesError,
     DbusClientUniqueResultError,
 )
-from dbus_python_client_gen import DPClientInvocationError, DPClientMethodCallContext
+from dbus_python_client_gen import (
+    DPClientInvocationError,
+    DPClientMethodCallContext,
+    DPClientSetPropertyContext,
+)
 
 from ._actions import BLOCKDEV_INTERFACE, FILESYSTEM_INTERFACE, POOL_INTERFACE
 from ._errors import (
     StratisCliActionError,
-    StratisCliAggregateError,
     StratisCliEngineError,
-    StratisCliEnginePropertyError,
     StratisCliIncoherenceError,
     StratisCliParserError,
     StratisCliStratisdVersionError,
+    StratisCliSynthUeventError,
     StratisCliUnknownInterfaceError,
     StratisCliUserError,
 )
@@ -108,28 +110,38 @@ def _interpret_errors_0(error):
     # control the contents or installation of "stratisd.conf"
     # and therefore, we cannot test this case reliably.
     if (
-        # pylint: disable=bad-continuation
-        error.get_dbus_name()
-        == "org.freedesktop.DBus.Error.AccessDenied"
+        error.get_dbus_name() == "org.freedesktop.DBus.Error.AccessDenied"
     ):  # pragma: no cover
         return (
             "Most likely stratis has insufficient permissions for the action requested."
         )
 
-    # We have observed two causes of this problem. The first is that
+    # We have observed three causes of this problem. The first is that
     # stratisd is not running at all. The second is that stratisd has not
-    # yet established its D-Bus service. While coverage of the first condition
-    # can be replicated by simply not starting stratisd, the second condition
-    # is difficult to replicate through testing.
+    # yet established its D-Bus service. The third is that stratisd is
+    # running with a new major version and is supplying a different name on the
+    # D-Bus than stratis is attempting to use. The second and third
+    # possibilities are both covered by a single error message.
     if error.get_dbus_name() == "org.freedesktop.DBus.Error.NameHasNoOwner":
-        for proc in psutil.process_iter():
-            try:
-                if proc.name() == "stratisd":  # pragma: no cover
-                    return "Most likely stratis is unable to connect to the stratis D-Bus service."
-            except psutil.NoSuchProcess:  # pragma: no cover
-                pass
+        try:
+            # pylint: disable=import-outside-toplevel
+            # isort: THIRDPARTY
+            import psutil
 
-        return "It appears that there is no stratisd process running."
+            for proc in psutil.process_iter():
+                try:
+                    if proc.name() == "stratisd":  # pragma: no cover
+                        return (
+                            "Most likely stratis is unable to connect to the "
+                            "stratisd D-Bus service."
+                        )
+                except psutil.NoSuchProcess:  # pragma: no cover
+                    pass
+
+            return "It appears that there is no stratisd process running."
+
+        except ImportError:  # pragma: no cover
+            return "Most likely there is no stratisd process running."
 
     # Due to the uncertain behavior with which libdbus
     # treats a timeout value of 0, it proves difficult to test this case,
@@ -137,9 +149,7 @@ def _interpret_errors_0(error):
     # Additional information may be found in the issue filed against libdbus
     # here: https://gitlab.freedesktop.org/dbus/dbus/issues/293
     if (
-        # pylint: disable=bad-continuation
-        error.get_dbus_name()
-        == "org.freedesktop.DBus.Error.NoReply"
+        error.get_dbus_name() == "org.freedesktop.DBus.Error.NoReply"
     ):  # pragma: no cover
         return (
             "stratis attempted communication with the daemon, stratisd, "
@@ -153,7 +163,6 @@ def _interpret_errors_0(error):
 
 
 def _interpret_errors_1(
-    # pylint: disable=bad-continuation
     errors,
 ):  # pylint: disable=too-many-return-statements, too-many-branches
     """
@@ -165,13 +174,12 @@ def _interpret_errors_1(
     """
     error = errors[0]
 
-    if (
-        # pylint: disable=bad-continuation
-        isinstance(error, DbusClientUniqueResultError)
-        and error.result == []
-    ):
-        fmt_str = "Most likely you specified a %s which does not exist."
-        return fmt_str % _interface_name_to_common_name(error.interface_name)
+    if isinstance(error, DbusClientUniqueResultError) and error.result == []:
+        return (
+            f"Most likely you specified a "
+            f"{_interface_name_to_common_name(error.interface_name)} "
+            f"which does not exist."
+        )
 
     # These errors can only arise if there is a bug in the way automatically
     # generated code is constructed, or if the introspection data from
@@ -179,46 +187,29 @@ def _interpret_errors_1(
     # daemon interface. This situation is unlikely and difficult to
     # elicit in a test.
     if isinstance(
-        # pylint: disable=bad-continuation
         error,
         (DbusClientMissingSearchPropertiesError, DbusClientMissingPropertyError),
     ):  # pragma: no cover
         return _DBUS_INTERFACE_MSG
 
     if isinstance(error, StratisCliEngineError):
-        fmt_str = (
-            "stratisd failed to perform the operation that you "
-            "requested. It returned the following information via "
-            "the D-Bus: %s."
+        return (
+            f"stratisd failed to perform the operation that you "
+            f"requested. It returned the following information via "
+            f"the D-Bus: {error}."
         )
-        return fmt_str % error
-
-    # This should arise only if the engine encounters an error while
-    # obtaining a property. Therefore, it is not tested.
-    if isinstance(error, StratisCliEnginePropertyError):  # pragma: no cover
-        return str(error)
 
     if isinstance(error, StratisCliParserError):
-        fmt_str = "You entered an invalid command: %s"
-        return fmt_str % error
+        return f"You entered an invalid command: {error}"
 
     if isinstance(error, StratisCliUserError):
-        fmt_str = "It appears that you issued an unintended command: %s"
-        return fmt_str % error
-
-    # The only situation in which an AggregateError can be raised is if there
-    # is a problem activating devcies, but the sim engine does not simulate
-    # activation of locked devices.
-    if isinstance(error, StratisCliAggregateError):  # pragma: no cover
-        fmt_str = "An iterative command generated one or more errors: %s"
-        return fmt_str % error
+        return f"It appears that you issued an unintended command: {error}"
 
     if isinstance(error, StratisCliStratisdVersionError):
-        fmt_str = (
-            "%s. stratis can execute only the subset of its "
-            "commands that do not require stratisd."
+        return (
+            f"{error}. stratis can execute only the subset of its "
+            f"commands that do not require stratisd."
         )
-        return fmt_str % error
 
     # An incoherence error should be pretty untestable. It can arise
     # * in the case of a stratisd bug. We would expect to fix that very
@@ -227,12 +218,17 @@ def _interpret_errors_1(
     # state while a command is being executed. This could be tested for,
     # but only with considerable difficulty, so we choose not to test.
     if isinstance(error, StratisCliIncoherenceError):  # pragma: no cover
-        fmt_str = (
-            "stratisd reported that it did not execute every action "
-            "that it would have been expected to execute as a result "
-            "of the command that you requested: %s"
+        return (
+            f"stratisd reported that it did not execute every action "
+            f"that it would have been expected to execute as a result "
+            f"of the command that you requested: {error}"
         )
-        return fmt_str % error
+
+    if isinstance(error, StratisCliSynthUeventError):
+        return (
+            f"stratis reported an error in generating a synethetic "
+            f"udev event: {error}"
+        )  # pragma: no cover
 
     # Some method calls may have an assignable underlying cause.
     # At present, automated testing of any of these assignable causes is
@@ -256,7 +252,7 @@ def _interpret_errors_1(
     return None  # pragma: no cover
 
 
-def _interpret_errors_2(errors):  # pragma: no cover
+def _interpret_errors_2(errors):
     """
     Interpret the error when it is known that the first error is a
     DPClientInvocationError
@@ -276,30 +272,33 @@ def _interpret_errors_2(errors):  # pragma: no cover
             # We do not test this error, as the only known way to cause it is
             # manipulation of selinux configuration, which is too laborious to
             # bother with at this time.
-            if next_error.get_dbus_name() == "org.freedesktop.DBus.Error.Disconnected":
+            if (
+                next_error.get_dbus_name() == "org.freedesktop.DBus.Error.Disconnected"
+            ):  # pragma: no cover
                 return (
                     "The D-Bus connection was disconnected during a "
                     "D-Bus interaction. Most likely, your selinux settings "
                     "prohibit that particular D-Bus interaction."
                 )
 
-            # We do not test this error, as the only known way to cause it is
-            # to spam the daemon with a succession of mutating commands from
-            # separate processes. The circumstances that cause this exception
-            # to be raised are a call to GetManagedObjects() that is initiated
-            # while a call that removes a filesystem or pool is in progress.
-            # In that case, the GetManagedObjects() call may process object
-            # paths that have not yet been removed and may encounter an error
-            # when calculating the object properties since the engine no longer
-            # has any record of the filesystem or pool.
             if next_error.get_dbus_name() == "org.freedesktop.DBus.Error.Failed":
                 context = error.context
+
+                # We do not test this error, as the only known way to cause it
+                # is to spam the daemon with a succession of mutating commands
+                # from separate processes. The circumstances that cause this
+                # exception to be raised are a call to GetManagedObjects() that
+                # is initiated while a call that removes a filesystem or pool
+                # is in progress. In that case, the GetManagedObjects() call
+                # may process object paths that have not yet been removed and
+                # may encounter an error when calculating the object properties
+                # since the engine no longer has any record of the filesystem
+                # or pool.
                 if (
-                    # pylint: disable=bad-continuation
                     error.interface_name == "org.freedesktop.DBus.ObjectManager"
                     and isinstance(context, DPClientMethodCallContext)
                     and context.method_name == "GetManagedObjects"
-                ):
+                ):  # pragma: no cover
                     return (
                         "A D-Bus method failed during execution of the "
                         "selected command. Most likely, the failure was due "
@@ -307,7 +306,17 @@ def _interpret_errors_2(errors):  # pragma: no cover
                         "and the command will succeed if run again."
                     )
 
-    return None
+                if isinstance(context, DPClientSetPropertyContext):
+                    return (
+                        f"stratisd failed to perform the operation that you "
+                        f"requested, because it could not set the D-Bus "
+                        f'property "{context.property_name}" belonging to '
+                        f'interface "{error.interface_name}" to "{context.value}". '
+                        f"It returned the following error: "
+                        f"{next_error.get_dbus_message()}."
+                    )
+
+    return None  # pragma: no cover
 
 
 def _interpret_errors(errors):
@@ -359,5 +368,5 @@ def handle_error(err):
         print(exit_msg, os.linesep, file=sys.stderr, flush=True)
         raise err
 
-    exit_msg = "Execution failed:%s%s" % (os.linesep, explanation)
+    exit_msg = f"Execution failed:{os.linesep}{explanation}"
     exit_(StratisCliErrorCodes.ERROR, exit_msg)
