@@ -15,9 +15,6 @@
 Error heirarchy for stratis cli.
 """
 
-# isort: STDLIB
-import os
-
 from ._stratisd_constants import (
     BLOCK_DEV_TIER_TO_NAME,
     STRATISD_ERROR_TO_NAME,
@@ -44,6 +41,42 @@ class StratisCliUserError(StratisCliRuntimeError):
     """
 
 
+class StratisCliNoPropertyChangeError(StratisCliUserError):
+    """
+    Raised when the user requests that a property be changed to its existing
+    value.
+    """
+
+    def __init__(self, value):
+        """
+        Initializer.
+
+        :param bool value: overprovision value
+        """
+        # pylint: disable=super-init-not-called
+        self.value = value
+
+
+class StratisCliOverprovisionChangeError(StratisCliNoPropertyChangeError):
+    """
+    Raised when the user requests the same overprovision state that the pool
+    already has.
+    """
+
+    def __str__(self):
+        return f"Pool's overprovision mode is already set to {str(self.value).lower()}"
+
+
+class StratisCliFsLimitChangeError(StratisCliNoPropertyChangeError):
+    """
+    Raised when the user requests the same FsLimti value that the pool already
+    has.
+    """
+
+    def __str__(self):
+        return f"Pool's filesystem limit is exactly {str(self.value).lower()}"
+
+
 class StratisCliResourceNotFoundError(StratisCliUserError):
     """
     Raised if a request was made to stratisd to update a resource
@@ -63,30 +96,8 @@ class StratisCliResourceNotFoundError(StratisCliUserError):
 
     def __str__(self):
         return (
-            "Command '%s' assumes that resource '%s' exists but it was not "
-            "found." % (self.command, self.resource)
-        )
-
-
-class StratisCliPropertyNotFoundError(StratisCliRuntimeError):
-    """
-    Exception raised when a property can not be found in the result of
-    a FetchProperties call. This can be due to a bug in our code or to
-    a property being unavailable via the FetchProperties interface.
-    """
-
-    def __init__(self, prop_name):
-        """ Initializer.
-
-            :param str prop_name: the property that did not exist
-        """
-        # pylint: disable=super-init-not-called
-        self.prop_name = prop_name
-
-    def __str__(self):
-        return (
-            "The requested property '%s' was not found in the FetchProperties "
-            "result for a D-Bus object" % self.prop_name
+            f"Command '{self.command}' assumes that resource '{self.resource}' "
+            f"exists but it was not found."
         )
 
 
@@ -99,15 +110,16 @@ class StratisCliPartialChangeError(StratisCliUserError):
     """
 
     def __init__(self, command, changed_resources, unchanged_resources):
-        """ Initializer.
+        """
+        Initializer.
 
-            :param str command: the command run that caused the error
-            :param changed_resources: the target resources that would change
-            :type changed_resources: frozenset of str
-            :param unchanged_resources: the target resources that would not change
-            :type unchanged_resources: frozenset of str
+        :param str command: the command run that caused the error
+        :param changed_resources: the target resources that would change
+        :type changed_resources: frozenset of str
+        :param unchanged_resources: the target resources that would not change
+        :type unchanged_resources: frozenset of str
 
-            Precondition: unchanged_resources != frozenset()
+        Precondition: unchanged_resources != frozenset()
         """
         # pylint: disable=super-init-not-called
         self.command = command
@@ -116,21 +128,21 @@ class StratisCliPartialChangeError(StratisCliUserError):
 
     def __str__(self):
         if len(self.unchanged_resources) > 1:
-            msg = "The '%s' action has no effect for resources %s" % (
-                self.command,
-                list(self.unchanged_resources),
+            msg = (
+                f"The '{self.command}' action has no effect for resources "
+                f"{list(self.unchanged_resources)}"
             )
         else:
-            msg = "The '%s' action has no effect for resource %s" % (
-                self.command,
-                list(self.unchanged_resources)[0],
+            msg = (
+                f"The '{self.command}' action has no effect for resource "
+                f"{list(self.unchanged_resources)[0]}"
             )
 
         if self.changed_resources != frozenset():
             if len(self.changed_resources) > 1:
-                msg += " but does for resources %s" % list(self.changed_resources)
+                msg += f" but does for resources {list(self.changed_resources)}"
             else:
-                msg += " but does for resource %s" % list(self.changed_resources)[0]
+                msg += f" but does for resource {list(self.changed_resources)[0]}"
 
         return msg
 
@@ -144,10 +156,11 @@ class StratisCliNoChangeError(StratisCliPartialChangeError):
     """
 
     def __init__(self, command, resource):
-        """ Initializer.
+        """
+        Initializer.
 
-            :param str command: the executed command
-            :param str resource: the target resource
+        :param str command: the executed command
+        :param str resource: the target resource
         """
         StratisCliPartialChangeError.__init__(
             self, command, frozenset(), frozenset([resource])
@@ -171,7 +184,40 @@ class StratisCliNameConflictError(StratisCliUserError):
         self.name = name
 
     def __str__(self):
-        return "A %s named %s already exists" % (self.object_type, self.name)
+        return f"A {self.object_type} named {self.name} already exists"
+
+
+class StratisCliParserError(StratisCliRuntimeError):
+    """
+    Raised when user has passed arguments that the parser should reject
+    but can not due to its configuration limitations.
+    """
+
+
+class StratisCliMissingClevisTangURLError(StratisCliParserError):
+    """
+    Raised during pool creation if --clevis=[tang|nbde] specified but no
+    URL supplied.
+    """
+
+    def __str__(self):
+        return (
+            "Specified binding with Clevis Tang server, but URL was not "
+            "specified. Use --tang-url option to specify tang URL."
+        )
+
+
+class StratisCliMissingClevisThumbprintError(StratisCliParserError):
+    """
+    Raised during pool creation if --clevis=[tang|nbde] specified but neither
+    a thumbprint nor instructions to trust the URL were supplied.
+    """
+
+    def __str__(self):
+        return (
+            "Specified binding with Clevis Tang server, but neither "
+            "--thumbprint nor --trust-url option was specified."
+        )
 
 
 class StratisCliIncoherenceError(StratisCliRuntimeError):
@@ -184,6 +230,12 @@ class StratisCliIncoherenceError(StratisCliRuntimeError):
     """
 
 
+class StratisCliSynthUeventError(StratisCliRuntimeError):
+    """
+    Raised if there was an error during a synthetic udev event generation.
+    """
+
+
 class StratisCliInUseError(StratisCliUserError):
     """
     Base class for if a request made of stratisd must result in a device being
@@ -199,15 +251,16 @@ class StratisCliInUseOtherTierError(StratisCliInUseError):
     """
 
     def __init__(self, pools_to_blockdevs, added_as):
-        """ Initializer.
+        """
+        Initializer.
 
-            :param pools_to_blockdevs: pools mapped to the blockdevs they own
-            :type pools_to_blockdevs: dict of str * frozenset of str
-            :param added_as: what tier the devices were to be added to
-            :type added_as: _stratisd_constants.BlockDevTiers
+        :param pools_to_blockdevs: pools mapped to the blockdevs they own
+        :type pools_to_blockdevs: dict of str * frozenset of str
+        :param added_as: what tier the devices were to be added to
+        :type added_as: _stratisd_constants.BlockDevTiers
 
-            Precondition: pools_to_blockdevs != {}
-            Precondition: all frozenset of str in pools_to_blockdevs have at least one item
+        Precondition: pools_to_blockdevs != {}
+        Precondition: all frozenset of str in pools_to_blockdevs have at least one item
         """
         # pylint: disable=super-init-not-called
         self.pools_to_blockdevs = pools_to_blockdevs
@@ -217,39 +270,29 @@ class StratisCliInUseOtherTierError(StratisCliInUseError):
         (target_blockdev_tier, already_blockdev_tier) = (
             BLOCK_DEV_TIER_TO_NAME(self.added_as),
             BLOCK_DEV_TIER_TO_NAME(
-                BlockDevTiers.Data
-                if self.added_as == BlockDevTiers.Cache
-                else BlockDevTiers.Cache
+                BlockDevTiers.DATA
+                if self.added_as == BlockDevTiers.CACHE
+                else BlockDevTiers.CACHE
             ),
         )
 
         msg = (
-            "At least one of the provided devices is already owned by an "
-            "existing pool's %s tier" % (already_blockdev_tier,)
+            f"At least one of the provided devices is already owned by an "
+            f"existing pool's {already_blockdev_tier} tier"
         )
 
         for pool_name, blockdevs in self.pools_to_blockdevs.items():
             if len(blockdevs) > 1:
                 msg += (
-                    "; devices %s would be added to the %s tier but are "
-                    "already in use in the %s tier of pool %s"
-                    % (
-                        list(blockdevs),
-                        target_blockdev_tier,
-                        already_blockdev_tier,
-                        pool_name,
-                    )
+                    f"; devices {list(blockdevs)} would be added to the {target_blockdev_tier} "
+                    f"tier but are already in use in the {already_blockdev_tier} tier "
+                    f"of pool {pool_name}"
                 )
             else:
                 msg += (
-                    "; device %s would be added to the %s tier but is "
-                    "already in use in the %s tier of pool %s"
-                    % (
-                        list(blockdevs)[0],
-                        target_blockdev_tier,
-                        already_blockdev_tier,
-                        pool_name,
-                    )
+                    "; device {list(blockdevs)[0]} would be added to the {target_blockdev_tier} "
+                    f"tier but is already in use in the {already_blockdev_tier} tier "
+                    f"of pool {pool_name}"
                 )
 
         return msg
@@ -262,15 +305,16 @@ class StratisCliInUseSameTierError(StratisCliInUseError):
     """
 
     def __init__(self, pools_to_blockdevs, added_as):
-        """ Initializer.
+        """
+        Initializer.
 
-            :param pools_to_blockdevs: pools mapped to the blockdevs they own
-            :type pools_to_blockdevs: dict of str * frozenset of str
-            :param added_as: what tier the devices were to be added to
-            :type added_as: _stratisd_constants.BlockDevTiers
+        :param pools_to_blockdevs: pools mapped to the blockdevs they own
+        :type pools_to_blockdevs: dict of str * frozenset of str
+        :param added_as: what tier the devices were to be added to
+        :type added_as: _stratisd_constants.BlockDevTiers
 
-            Precondition: pools_to_blockdevs != {}
-            Precondition: all frozenset of str in pools_to_blockdevs have at least one item
+        Precondition: pools_to_blockdevs != {}
+        Precondition: all frozenset of str in pools_to_blockdevs have at least one item
         """
         # pylint: disable=super-init-not-called
         self.pools_to_blockdevs = pools_to_blockdevs
@@ -280,22 +324,20 @@ class StratisCliInUseSameTierError(StratisCliInUseError):
         blockdev_tier = BLOCK_DEV_TIER_TO_NAME(self.added_as)
 
         msg = (
-            "At least one of the provided devices is already owned by an "
-            "existing pool's %s tier" % (blockdev_tier,)
+            f"At least one of the provided devices is already owned by an "
+            f"existing pool's {blockdev_tier} tier"
         )
 
         for pool_name, blockdevs in self.pools_to_blockdevs.items():
             if len(blockdevs) > 1:
                 msg += (
-                    "; devices %s would be added to the %s tier but are "
-                    "already in use in the %s tier of pool %s"
-                    % (list(blockdevs), blockdev_tier, blockdev_tier, pool_name)
+                    f"; devices {list(blockdevs)} would be added to the {blockdev_tier} tier "
+                    f"but are already in use in the {blockdev_tier} tier of pool {pool_name}"
                 )
             else:
                 msg += (
-                    "; device %s would be added to the %s tier but is "
-                    "already in use in the %s tier of pool %s"
-                    % (list(blockdevs)[0], blockdev_tier, blockdev_tier, pool_name)
+                    f"; device {list(blockdevs)[0]} would be added to the {blockdev_tier} tier "
+                    f"but is already in use in the {blockdev_tier} tier of pool {pool_name}"
                 )
 
         return msg
@@ -317,7 +359,7 @@ class StratisCliKeyfileNotFoundError(StratisCliUserError):
         self.keyfile_path = keyfile_path
 
     def __str__(self):
-        return 'There is no keyfile at the path "%s"' % self.keyfile_path
+        return f'There is no keyfile at the path "{self.keyfile_path}"'
 
 
 class StratisCliUnknownInterfaceError(StratisCliRuntimeError):
@@ -326,43 +368,16 @@ class StratisCliUnknownInterfaceError(StratisCliRuntimeError):
     """
 
     def __init__(self, interface_name):
-        """ Initializer.
+        """
+        Initializer.
 
-            :param str interface_name: the unexpected interface name
+        :param str interface_name: the unexpected interface name
         """
         # pylint: disable=super-init-not-called
         self._interface_name = interface_name
 
     def __str__(self):
-        return "unexpected interface name %s" % self._interface_name
-
-
-class StratisCliEnginePropertyError(StratisCliRuntimeError):
-    """
-    Raised if there was a failure to obtain a property due to an error in
-    stratisd's engine.
-
-    In this case, stratisd does not return an error code, but an error message
-    is still transmitted.
-    """
-
-    def __init__(self, prop_name, message):
-        """ Initializer.
-
-            :param str prop_name: the property that could not be obtained
-            :param str message: the error message returned by the engine in
-                                place of the property
-        """
-        # pylint: disable=super-init-not-called
-        self.prop_name = prop_name
-        self.message = message
-
-    def __str__(self):
-        return (
-            "stratisd encountered the following error while obtaining the "
-            "requested property '%s' via the FetchProperties interface for a "
-            "D-Bus object: %s"
-        ) % (self.prop_name, self.message)
+        return f"unexpected interface name {self._interface_name}"
 
 
 class StratisCliEngineError(StratisCliRuntimeError):
@@ -371,17 +386,18 @@ class StratisCliEngineError(StratisCliRuntimeError):
     """
 
     def __init__(self, rc, message):
-        """ Initializer.
+        """
+        Initializer.
 
-            :param rc int: the error code returned by the engine
-            :param str message: whatever message accompanied the error code
+        :param rc int: the error code returned by the engine
+        :param str message: whatever message accompanied the error code
         """
         # pylint: disable=super-init-not-called
         self.error_code = rc
         self.message = message
 
     def __str__(self):
-        return "%s: %s" % (STRATISD_ERROR_TO_NAME(self.error_code), self.message)
+        return f"{STRATISD_ERROR_TO_NAME(self.error_code)}: {self.message}"
 
 
 class StratisCliActionError(StratisCliRuntimeError):
@@ -451,35 +467,6 @@ class StratisCliStratisdVersionError(StratisCliRuntimeError):
         )
 
 
-class StratisCliAggregateError(StratisCliRuntimeError):
-    """
-    Raised when multiple errors have occured in a looping operation.
-    """
-
-    def __init__(self, operation, type_, errors):
-        """
-        Initializer.
-        :param str operation: the looping operation that failed.
-        :param str type_: the type of the resource for which the operation failed.
-        :param errors: list of all errors that occurred during the looping operation.
-        :type errors: list of Exception
-        """
-        # pylint: disable=super-init-not-called
-        self.operation = operation
-        self.type = type_
-        self.errors = errors
-
-    def __str__(self):
-        return (
-            "The operation '%s' on a resource of type %s failed. The following "
-            "errors occurred:\n%s"
-        ) % (
-            self.operation,
-            self.type,
-            os.linesep.join([str(error) for error in self.errors]),
-        )
-
-
 class StratisCliPartialFailureError(StratisCliRuntimeError):
     """
     A non-fatal error to be reported at the end as part of a StratisCliAggregateError.
@@ -501,10 +488,7 @@ class StratisCliPartialFailureError(StratisCliRuntimeError):
         self.error_message = error_message
 
     def __str__(self):
-        fmt_str = 'Partial action "%s" failed for %s' % (
-            self.action,
-            self.identifier_string,
-        )
+        fmt_str = f'Partial action "{self.action}" failed for {self.identifier_string}'
         if self.error_message is not None:
-            fmt_str += ": %s" % self.error_message
+            fmt_str += f": {self.error_message}"
         return fmt_str
