@@ -16,6 +16,8 @@ Definition of filesystem actions to display in the CLI.
 """
 
 from .._actions import LogicalActions
+from ._debug import FILESYSTEM_DEBUG_SUBCMDS
+from ._range import RangeAction
 
 LOGICAL_SUBCMDS = [
     (
@@ -31,6 +33,14 @@ LOGICAL_SUBCMDS = [
                         nargs="+",
                     ),
                 ),
+                (
+                    "--size",
+                    dict(
+                        action=RangeAction,
+                        dest="size",
+                        help='The size of the filesystems to be created, e.g., "32GiB"',
+                    ),
+                ),
             ],
             func=LogicalActions.create_volumes,
         ),
@@ -98,4 +108,11 @@ LOGICAL_SUBCMDS = [
             func=LogicalActions.rename_fs,
         ),
     ),
+    (
+        "debug",
+        dict(
+            help=("Miscellaneous filesystem-level debug commands"),
+            subcmds=FILESYSTEM_DEBUG_SUBCMDS,
+        ),
+    ),
 ]
