@@ -16,14 +16,27 @@ Definition of block device actions to display in the CLI.
 """
 
 from .._actions import PhysicalActions
+from ._debug import BLOCKDEV_DEBUG_SUBCMDS
 
 PHYSICAL_SUBCMDS = [
-    ('list',
-     dict(
-         help="List information about blockdevs in the pool",
-         args=[
-             ('pool_name',
-              dict(action='store', default=None, nargs="?", help='Pool name')),
-         ],
-         func=PhysicalActions.list_pool)),
+    (
+        "list",
+        dict(
+            help="List information about blockdevs in the pool",
+            args=[
+                (
+                    "pool_name",
+                    dict(action="store", default=None, nargs="?", help="Pool name"),
+                )
+            ],
+            func=PhysicalActions.list_devices,
+        ),
+    ),
+    (
+        "debug",
+        dict(
+            help=("Miscellaneous blockdev-level debug commands"),
+            subcmds=BLOCKDEV_DEBUG_SUBCMDS,
+        ),
+    ),
 ]
